addBookCard = function(title, author, coverurl, bookurl, bookid) {
    $("#booklist").append($.parseHTML('<a class="ui book card" bookid="' + bookid + '" lastread="' + (localStorage.getItem("bookreader.book." + bookid + ".lastread") || "0") + '" href="reader.html?bookurl=' + btoa(bookurl) + '"><div class="image"><img src="' + coverurl + '" class="visible content"></div><div class="content"><span class="header">' + title + '</span><span class="meta author">' + author + '</span></div></a>'));
}

addBookCardFromFile = function(filename, sortAfterAdd) {
    return new Promise(function(resolve, reject) {
        getBookMetadata(filename).then(function(meta) {
            addBookCard(meta.title, meta.author, meta.coverurl, meta.bookurl, meta.bookid);
            if (sortAfterAdd) {
                sortBookList();
            }
        }, function(err) {
            rejectCleanup(err);
        });
        resolve();

        function rejectCleanup(err) {
            reject(err);
        }
    });
}

sortBookList = function() {
    var list = $("#booklist a").get();
    list = list.sort(function(a, b) {
        var ad = parseInt(a.getAttribute("lastread") || new Date().getTime());
        var bd = parseInt(b.getAttribute("lastread") || new Date().getTime());
        return bd - ad;
    });
    for (var i = 0; i < list.length; i++) {
        list[i].parentNode.appendChild(list[i]);
    }
    console.log("sorted");
}

refreshBookList = function() {
    $("#booklist").html("<div class=\"ui message\"><div class=\"header\">No books found</div><p>No ePub books were found in " + getAndCreateBooksDir() + " . You can open your file manager and put your books in the folder then click the refresh button to refresh the booklist, or click on the <i class=\"add icon\"></i> button on the toolbar at the top to add a book to your library. <br> <a class=\"ui compact tiny button\" href=\"javascript:openBooksFolder();\">Open file manager</a> <a class=\"ui compact tiny button\" href=\"javascript:refreshBookList();\">Refresh book list</a></p></div>");

    var bf = getBookFilenames();
    if (bf.length > 0) {
        $("#booklist").empty();
        clearMetaCache();
    }
    for (var i = 0; i < bf.length; i++) {
        console.log(bf[i]);
        addBookCardFromFile(bf[i], (((i + 1) == bf.length) ? true : false));
    }
}

addBookFiles = function() {
    var bc = $('#bookchooser');
    bc.removeAttr('value');
    bc.unbind("change");
    bc.change(function() {
        $("#bookadderloader").addClass("active");
        var bfiles = $(this).val().split(";");
        console.log(bfiles)
        for (var i = 0; i < bfiles.length; i++) {
            if (!bfiles[i].endsWith(".epub")) {
                alert("Please choose ePub files only")
                $("#bookadderloader").removeClass("active");
                return;
            }
        }
        for (var i = 0; i < bfiles.length; i++) {
            var ii = i;
            var fname = bfiles[i];
            copyFile(fname, path.join(getAndCreateBooksDir(), path.basename(fname))).then(function() {
                if (ii + 1 == bfiles.length) {
                    console.log("done");
                    $("#bookadderloader").removeClass("active");
                    location.reload();
                }
            });
        }
    });
    bc.trigger("click");
}

initSearch = function() {
    var searchtimeout;
    var updateSearchResults = function() {
        window.clearTimeout(searchtimeout);
        if ($("#booksearch").val().trim() == "") {
            $("#booklist .book").css("display", "block");
        } else {
            $("#booklist .book:containsi('" + $("#booksearch").val().trim() + "')").css("display", "block");
            $("#booklist .book:not(:containsi('" + $("#booksearch").val().trim() + "'))").css("display", "none");
        }
    }
    $("#booksearch").keyup(function(e) {
        window.clearTimeout(searchtimeout);
        searchtimeout = window.setTimeout(function() {
            updateSearchResults();
        }, 500);
        if (e.keyCode == 13) {
            window.clearTimeout(searchtimeout);
            updateSearchResults();
        }
    });
    $("#booksearch").change(function() {
        window.clearTimeout(searchtimeout);
        updateSearchResults();
    });
};

(function init() {
    var a = (nw.App.argv.pop() || "").replace("~/", os.homedir() + "/");
    if (a == undefined || a == null || sessionStorage.getItem("noargs") !== null) {} else {
        if (fsExists(a) && a.endsWith(".epub")) {
            location = "reader.html?bookurl=" + btoa("file://" + a);
        }
    }
    try {
      if (nw.App.argv.pop().endsWith("updated") && sessionStorage.getItem("updatemsgshown") === null) {
        alert("BookReader has been updated sucessfully")
        sessionStorage.setItem("updatemsgshown", "yes");
      }
    } catch (e) {}
    try {
      if (sessionStorage.getItem("newversionshown") === null) {
        checkNewVersionShowNotification();
        sessionStorage.setItem("newversionshown", "yes");
      }
    } catch (e) {}
    refreshBookList();
    initSearch();
    $("#lastbook").css("display", "none");
    if (localStorage.getItem("bookreader.global.lastread.filename") !== null) {
        if (fsExists(localStorage.getItem("bookreader.global.lastread.filename").replace("file://", ""))) {
            getBookMetadata(localStorage.getItem("bookreader.global.lastread.filename").replace("file://", "")).then(function(meta) {
              $("#lastbook").css("display", "inline-block");
                $("#lastbook").text("Open last book: " + meta.title)
                $("#lastbook").attr("href", "reader.html?bookurl=" + btoa(localStorage.getItem("bookreader.global.lastread.filename")));
            });
        } else {
            $("#lastbook").css("display", "none");
        }
    } else {
        $("#lastbook").css("display", "none");
    }
    setTimeout(sortBookList, 1000);
})();
