bookFolder = "";
fs = require("fs");
os = require("os");
path = require("path");
spawn = require('child_process').spawn;

defaultSettings = {
    "family": "Georgia, serif",
    "size": "medium",
    "colorscheme": "solarizedLight",
    "lineheight": "2",
    "margin": "1em"
}

colorSchemes = {
    "white": {
        "bg": "white",
        "fg": "black"
    },
    "black": {
        "bg": "black",
        "fg": "white"
    },
    "darkgray": {
        "bg": "rgb(64,64,64)",
        "fg": "rgb(220,220,220)"
    },
    "sepia": {
        "bg": "wheat",
        "fg": "black"
    },
    "solarizedDark": {
        "bg": "#002b36",
        "fg": "#839496"
    },
    "solarizedLight": {
        "bg": "#fdf6e3",
        "fg": "#657b83"
    }
}

getSetting = function(sname) {
    return new Promise(function(resolve, reject) {
        try {
            if (localStorage.getItem("bookreader.settings." + sname) === null) {
                localStorage.setItem("bookreader.settings." + sname, defaultSettings[sname]);
            }
            resolve(localStorage.getItem("bookreader.settings." + sname));
        } catch (e) {
            reject(e);
        }
    });
}

getSettingsCSSObject = function(useimportant) {
    var useimportant = useimportant || false;
    var suf = ""
    if (useimportant) {
        suf = " !important";
    }
    return new Promise(function(resolve, reject) {
        var r = {};
        getSetting("family").then(function(val) {
            r["font-family"] = val + suf;
            getSetting("size").then(function(val) {
                r["font-size"] = val + suf;
                getSetting("colorscheme").then(function(val) {
                    r["background"] = colorSchemes[val].bg + suf;
                    r["color"] = colorSchemes[val].fg + suf;
                    getSetting("lineheight").then(function(val) {
                        r["line-height"] = val + suf;
                        getSetting("margin").then(function(val) {
                            r["padding"] = val + suf;
                            resolve(r);
                        });
                    });
                });
            });
        });
    });
}

getSettingsCSS = function(useimportant) {
    return new Promise(function(resolve, reject) {
        getSettingsCSSObject(useimportant).then(function(obj) {
            var cssstr = "";
            $.each(obj, function(key, value) {
                cssstr += key + ":" + value + ";";
            });
            resolve(cssstr);
        });
    });
}

setSetting = function(sname, svalue) {
    localStorage.setItem("bookreader.settings." + sname, svalue)
}

getBookMetadata = function(filename, forceupdate) {
    var fu = forceupdate || false;
    return new Promise(function(resolve, reject) {
        var cp = "bookreader.metacache." + filename;
        var ccp = "bookreader.covercache.";
        if (localStorage.getItem(cp + ".title") === null || fu == true) {
            try {
                var Book = ePub("file://" + filename);
            } catch (e) {
                rejectCleanup(e);
            }
            Book.getMetadata().then(function(meta) {
                localStorage.setItem(cp + ".title", meta.bookTitle);
                localStorage.setItem(cp + ".author", meta.creator);
                localStorage.setItem(cp + ".description", meta.description);
                localStorage.setItem(cp + ".bookid", btoa((localStorage.getItem(cp + ".title") || "") + ":" + (localStorage.getItem(cp + ".author") || "")));
                if ("bookreader.book." + localStorage.getItem(cp + ".bookid") + ".lastread" === null) {
                    localStorage.setItem("bookreader.book." + localStorage.getItem(cp + ".bookid") + ".lastread", new Date().getTime());
                }
                getCoverAsDataURL(Book).then(function(coverurl) {
                    localStorage.setItem(ccp + localStorage.getItem(cp + ".bookid"), coverurl);
                    resolve({
                        "title": localStorage.getItem(cp + ".title") || "",
                        "author": localStorage.getItem(cp + ".author") || "",
                        "bookid": localStorage.getItem(cp + ".bookid"),
                        "description": localStorage.getItem(cp + ".description"),
                        "filename": filename,
                        "coverurl": localStorage.getItem(ccp + localStorage.getItem(cp + ".bookid")) || "cover.png",
                        "bookurl": "file://" + filename
                    });
                }, function() {
                    localStorage.setItem(ccp + localStorage.getItem(cp + ".bookid"), "cover.png");
                    resolve({
                        "title": localStorage.getItem(cp + ".title") || "",
                        "author": localStorage.getItem(cp + ".author") || "",
                        "bookid": localStorage.getItem(cp + ".bookid"),
                        "description": localStorage.getItem(cp + ".description"),
                        "filename": filename,
                        "coverurl": localStorage.getItem(ccp + localStorage.getItem(cp + ".bookid")) || "cover.png",
                        "bookurl": "file://" + filename
                    });
                });
            });
        } else {
            resolve({
                "title": localStorage.getItem(cp + ".title") || "",
                "author": localStorage.getItem(cp + ".author") || "",
                "bookid": localStorage.getItem(cp + ".bookid"),
                "description": localStorage.getItem(cp + ".description"),
                "filename": filename,
                "coverurl": localStorage.getItem(ccp + localStorage.getItem(cp + ".bookid")) || "cover.png",
                "bookurl": "file://" + filename
            });
        }

        function rejectCleanup(err) {
            reject(err);
        }
    });
}

var fileManagerCommandForPath = function(pathToOpen, isFile) {
    var args, command;
    switch (process.platform) {
        case 'darwin':
            return {
                command: 'open',
                label: 'Finder',
                args: ['-R', pathToOpen]
            };
        case 'win32':
            args = ["/n,/e," + pathToOpen];
            if (process.env.SystemRoot) {
                command = path.join(process.env.SystemRoot, 'explorer.exe');
            } else {
                command = 'explorer.exe';
            }
            return {
                command: command,
                label: 'Explorer',
                args: args
            };
        default:
            if (isFile) {
                pathToOpen = path.dirname(pathToOpen);
            }
            return {
                command: 'xdg-open',
                label: 'File Manager',
                args: [pathToOpen]
            };
    }
}

openBooksFolder = function() {
    var fc = fileManagerCommandForPath(getAndCreateBooksDir());
    spawn(fc.command, fc.args).unref();
}

function copyFile(source, target) {
    return new Promise(function(resolve, reject) {
        var rd = fs.createReadStream(source);
        rd.on('error', rejectCleanup);
        var wr = fs.createWriteStream(target);
        wr.on('error', rejectCleanup);

        function rejectCleanup(err) {
            rd.destroy();
            wr.end();
            reject(err);
        }
        wr.on('finish', resolve);
        rd.pipe(wr);
    });
}

clearMetaCache = function() {
    var arr = []; // Array to hold the keys
    // Iterate over localStorage and insert the keys that meet the condition into arr
    for (var i = 0; i < localStorage.length; i++) {
        if (localStorage.key(i).startsWith("bookreader.covercache") || localStorage.key(i).startsWith("bookreader.metacache")) {
            arr.push(localStorage.key(i));
        }
    }

    // Iterate over arr and remove the items by key
    for (var i = 0; i < arr.length; i++) {
        localStorage.removeItem(arr[i]);
    }
}

clearPosAndStats = function() {
    var arr = []; // Array to hold the keys
    // Iterate over localStorage and insert the keys that meet the condition into arr
    for (var i = 0; i < localStorage.length; i++) {
        if (localStorage.key(i).startsWith("bookreader.book.") || localStorage.key(i).startsWith("bookreader.globalstats") || localStorage.key(i).startsWith("bookreader.stats")) {
            arr.push(localStorage.key(i));
        }
    }

    // Iterate over arr and remove the items by key
    for (var i = 0; i < arr.length; i++) {
        localStorage.removeItem(arr[i]);
    }
}

clearSettings = function() {
    var arr = []; // Array to hold the keys
    // Iterate over localStorage and insert the keys that meet the condition into arr
    for (var i = 0; i < localStorage.length; i++) {
        if (localStorage.key(i).startsWith("bookreader.settings.") || localStorage.key(i).startsWith("bookreader.setting.")) {
            arr.push(localStorage.key(i));
        }
    }

    // Iterate over arr and remove the items by key
    for (var i = 0; i < arr.length; i++) {
        localStorage.removeItem(arr[i]);
    }
}

fsExists = function(path) {
    try {
        fs.accessSync(path, fs.F_OK);
        return true;
    } catch (e) {
        return false;
    }
}

mkdir = function(path) {
    try {
        fs.mkdirSync(path);
    } catch (e) {
        if (e.code != 'EEXIST') throw e;
    }
}

isPortable = function() {
    if (fsExists(process.execPath + "/../portablemode") || fsExists(nw.global.__dirname + "/portablemode")) {
        return true;
    } else {
        return false;
    }
}

getFiles = function(dir, files_) {
    files_ = files_ || [];
    var files = fs.readdirSync(dir);
    for (var i in files) {
        var name = dir + '/' + files[i];
        if (fs.statSync(name).isDirectory()) {
            getFiles(name, files_);
        } else {
            files_.push(name);
        }
    }
    return files_;
}

getAndCreateBooksDir = function() {
    var os = require("os");
    if (isPortable()) {
        mkdir(path.resolve(process.execPath + "/../../Books"));
        return path.resolve(process.execPath + "/../../Books");
    } else {
        mkdir(path.resolve(os.homedir() + "/Documents"));
        mkdir(path.resolve(os.homedir() + "/Documents/Books"));
        return path.resolve(os.homedir() + "/Documents/Books");
    }
}

getBookFilenames = function() {
    var f = getFiles(getAndCreateBooksDir());
    var ff = [];
    for (var i = 0; i < f.length; i++) {
        if (f[i].endsWith(".epub")) {
            ff.push(f[i]);
        }
    }
    return ff;
}

getCoverAsDataURL = function(book, callback) {
    return new Promise(function(resolve, reject) {
        try {
            book.coverUrl().then(function(blobUrl) {
                var xhr = new XMLHttpRequest;
                xhr.responseType = 'blob';
                xhr.onload = function() {
                    var recoveredBlob = xhr.response;
                    var reader = new FileReader;
                    reader.onload = function() {
                        resolve(reader.result);
                    };
                    reader.readAsDataURL(recoveredBlob);
                };
                xhr.open('GET', blobUrl);
                xhr.send();
            }, reject);
        } catch (e) {
            reject(e);
        }
    });
}

setPageTitle = function(title) {
    $(".pagetitle").text(title);
    $("title").text(title);
}

function getUrlVars() {
    var vars = {};
    var parts = window.location.href.replace(/[?&]+([^=&]+)=([^&]*)/gi, function(m, key, value) {
        vars[key] = value;
    });
    return vars;
}

nw.App.on('open', function(cmdline) {
    var a = cmdline.split("package.nw")[1].trim().replace("\\ ", "").replace("~/", os.homedir() + "/");
    if (a == undefined || a == null) {} else {
        if (fsExists(a) && a.endsWith(".epub")) {
            window.location = "reader.html?bookurl=" + btoa("file://" + a);
        }
    }
});



/*    0 if the versions are equal
 *    a negative integer if v1 < v2
 *    a positive integer if v1 > v2
 */
function versionCompare(v1, v2, options) {
    var v1parts = v1.split(/[.-]/);
    var v2parts = v2.split(/[.-]/);

    function compareParts(v1parts, v2parts, options) {
        var zeroExtend = options && options.zeroExtend;

        if (zeroExtend) {
            while (v1parts.length < v2parts.length) v1parts.push("0");
            while (v2parts.length < v1parts.length) v2parts.push("0");
        }

        for (var i = 0; i < v1parts.length; ++i) {
            if (v2parts.length == i) {
                return 1;
            }

            var v1part = parseInt(v1parts[i]);
            var v2part = parseInt(v2parts[i]);
            // (NaN == NaN) -> false
            var v1part_is_string = !(v1part == v1part);
            var v2part_is_string = !(v2part == v2part);
            v1part = v1part_is_string ? v1parts[i] : v1part;
            v2part = v2part_is_string ? v2parts[i] : v2part;

            if (v1part_is_string == v2part_is_string) {
                if (v1part_is_string == false) {
                    // integer compare
                    if (v1part == v2part) {
                        continue;
                    } else if (v1part > v2part) {
                        return 1;
                    } else {
                        return -1;
                    }
                } else {
                    // letters and numbers in string
                    // split letters and numbers
                    var v1subparts = v1part.match(/[a-zA-Z]+|[0-9]+/g);
                    var v2subparts = v2part.match(/[a-zA-Z]+|[0-9]+/g);
                    if ((v1subparts.length == 1) && (v2subparts.length == 1)) {
                        // only letters in string
                        v1part = v1subparts[0];
                        v2part = v2subparts[0];
                        if (v1part == v2part) {
                            continue;
                        } else if (v1part > v2part) {
                            return 1;
                        } else {
                            return -1;
                        }
                    }
                    var result = compareParts(v1subparts, v2subparts);
                    if (result == 0) {
                        continue;
                    } else {
                        return result;
                    }
                }
            } else {
                return v2part_is_string ? 1 : -1;
            }
        }

        if (v1parts.length != v2parts.length) {
            return -1;
        }

        return 0;
    }

    return compareParts(v1parts, v2parts, options);
}

request = function(method, url) {
    return new Promise(function(resolve, reject) {
        var xhr = new XMLHttpRequest();
        xhr.open(method, url);
        xhr.onload = function() {
            if (this.status >= 200 && this.status < 300) {
                resolve(xhr.response);
            } else {
                reject({
                    status: this.status,
                    statusText: xhr.statusText
                });
            }
        };
        xhr.onerror = function() {
            reject({
                status: this.status,
                statusText: xhr.statusText
            });
        };
        xhr.send();
    });
}

getOwnPackageJSON = function() {
    return JSON.parse(fs.readFileSync(nw.global.__dirname + "/package.json", 'utf8'));
}

isNewVersionAvailable = function() {
    return new Promise(function(resolve, reject) {
        var opj = {};
        var spj = {};
        var vcr = null;
        try {
            opj = getOwnPackageJSON();
        } catch (e) {
            console.warn("Cannot get own package.json ", e)
            reject(e);
        }

        try {
            request("GET", (opj.updateCheckURL || "http://geek1011.github.io/BookReader/Updates/update.json") + "?t=" + String(new Date().getTime())).then(function(r) {
                try {
                    spj = JSON.parse(r);
                    vcr = versionCompare(opj.version, spj.version);
                    if (vcr == -1) {
                        console.log("New version available", opj, spj, vcr);
                        resolve({
                            "newVersion": true,
                            "versionJSON": spj,
                            "ownJSON": opj
                        });
                    } else if (vcr == 0) {
                        console.log("No new version available", opj, spj, vcr);
                        resolve({
                            "newVersion": false,
                            "versionJSON": spj,
                            "ownJSON": opj
                        });
                    } else if (vcr == 1) {
                        console.warn("Current version newer than server. Possible error.", opj, spj, vcr);
                        resolve({
                            "newVersion": false,
                            "versionJSON": spj,
                            "ownJSON": opj
                        });
                    }
                } catch (e) {
                    console.warn("Cannot parse update JSON");
                    reject(e);
                }
            }, reject);
        } catch (e) {
            console.warn("Cannot get " + (opj.updateCheckURL || "http://geek1011.github.io/BookReader/Updates/update.json") + "?t=" + String(new Date().getTime()), e)
            reject(e);
        }
    });
}

checkNewVersionShowNotification = function() {
    isNewVersionAvailable().then(function(r) {
        if (r.newVersion) {
            var notification = new Notification('BookReader update available', {
                icon: 'update.png',
                body: "BookReader " + String(r.versionJSON.version) + " is now available. Click this notification to install it now.",
            }).onclick = function(e) {
                window.location = "applyupdate.html"
                e.target.close();
            };
        }
    });
}

init = function() {
    if (localStorage.getItem("bookreader.version.new") == null) {
        localStorage.clear();
        localStorage.setItem("bookreader.version.new", "true");
        location.reload();
    }
    jQuery.expr[':'].containsi = function(a, i, m) {
        var sText = (a.textContent || a.innerText || "");
        var zRegExp = new RegExp(m[3], 'i');
        return zRegExp.test(sText);
    }

    $('.ui.checkbox').checkbox();

    bookFolder = getAndCreateBooksDir();
}
init();
