book = null;
bookfilename = null;
cachedbookmeta = null;
pageset = false;

EPUBJS.Hooks.register('beforeChapterDisplay').swipeDetection = function (callback, renderer) {
    function detectSwipe() {
        var script = renderer.doc.createElement('script');
        script.text = "\
      var swiper = new Hammer(document);\
      swiper.on('swipeleft', function() {\
        parent.Book.nextPage();\
      });\
      swiper.on('swiperight', function() {\
        parent.Book.prevPage();\
      });";
        renderer.doc.head.appendChild(script);
    }
    EPUBJS.core.addScript('http://geek1011.github.io/ePubViewer/epubjs/libs/hammer.min.js', detectSwipe, renderer.doc.head);
    if (callback) {
        callback();
    }
};

EPUBJS.Hooks.register('beforeChapterDisplay').linkColorFix = function (callback, renderer) {
    try {
        var se = document.createElement("style");
        se.innerText = "a:link, a:visited {color: black !important;text-decoration: underline !important;}";
        render.doc.body.appendChild(se);
    } catch (e) {
        console.error("Failed to fix link colors");
    }
    if (callback) {
        callback();
    }
};

EPUBJS.Hooks.register("beforeChapterDisplay").pageTurns = function (callback, renderer) {

    var lock = false;
    var arrowKeys = function (e) {
        e.preventDefault();
        if (lock) return;

        if (e.keyCode == 37) {
            book.prevPage();
            lock = true;
            setTimeout(function () {
                lock = false;
            }, 100);
            return false;
        }

        if (e.keyCode == 39) {
            book.nextPage();
            lock = true;
            setTimeout(function () {
                lock = false;
            }, 100);
            return false;
        }

    };
    renderer.doc.addEventListener('keydown', arrowKeys, false);
    if (callback) callback();
}

applyBookCSS = function () {
    getSettingsCSSObject(false).then(function (obj) {
        $.each(obj, function (k, v) {
            book.setStyle(k, v);
        });
    });
    getSettingsCSS(true).then(function (css) {
        $(".ui.menu.top").attr("style", css.replace("margin", "-ignore-margin").replace("padding", "-ignore-padding").replace("font-size", "-ignore-font-size"));
        $(".ui.menu .item").attr("style", "color: inherit;")
        $("#bookcontainer").attr("style", css.replace("margin", "-ignore-margin").replace("padding", "-ignore-padding").replace("font-size", "-ignore-font-size"));
    });
}

arrayContains = function (a, obj) {
    for (var i = 0; i < a.length; i++) {
        if (a[i] === obj) {
            return true;
        }
    }
    return false;
}

isBookmarked = function (cfi) {
    var bl = (localStorage.getItem("bookreader.book." + cachedbookmeta.bookid + ".bookmarks") || "").split("|");
    return arrayContains(bl, cfi);
}

updateBookmarkIndicator = function () {
    if (isBookmarked(book.getCurrentLocationCfi())) {
        $("#bookmarkicon").removeClass("remove");
    } else {
        $("#bookmarkicon").addClass("remove");
    }
}

toggleBookmark = function () {
    if (isBookmarked(book.getCurrentLocationCfi())) {
        console.log("removing bookmark");
        localStorage.setItem("bookreader.book." + cachedbookmeta.bookid + ".bookmarks", (localStorage.getItem("bookreader.book." + cachedbookmeta.bookid + ".bookmarks") || "").replace(book.getCurrentLocationCfi() + "|", ""));
        $("#bookmarkicon").addClass("remove");
    } else {
        console.log("adding bookmark");
        localStorage.setItem("bookreader.book." + cachedbookmeta.bookid + ".bookmarks", (localStorage.getItem("bookreader.book." + cachedbookmeta.bookid + ".bookmarks") || "") + book.getCurrentLocationCfi() + "|");
        $("#bookmarkicon").removeClass("remove");
    }
}

updateActiveToc = function () {
    $("#toc a.item").each(function (k, v) {
        if ($(v).attr("data-url") == "epubcfi(" + book.currentChapter.cfiBase + ")") {
            $(v).addClass("active");
        } else {
            $(v).removeClass("active");
        }
    });
}

updateBookmarksList = function () {
    var bl = (localStorage.getItem("bookreader.book." + cachedbookmeta.bookid + ".bookmarks") || "").split("|");
    var nl = $("#bookmarks");
    nl.empty();
    $.each(bl, function (k, v) {
        if (v !== "") {
            nl.append(
                $("<a class=\"item\" data-url=\"" + v + "\">" + ("Bookmark " + String(k + 1) + " - " + String((book.locations.percentageFromCfi(v) * 100).toFixed(2)) + "% into book") + "</a>")
                .unbind("click")
                .click(function () {
                    var u = $(this).attr("data-url");
                    console.log(u);
                    $('#tocbookmarksmodal').modal('hide');
                    // It crashes and uses up all cpu and memory if page changed while modal is open
                    setTimeout(function () {
                        book.goto(u);
                    }, 900);
                })
            );
        }
    });
}

updatePercentageIndicator = function () {
    $(".curpercent").text(String((book.locations.percentageFromCfi(book.getCurrentLocationCfi()) * 100).toFixed(0) + "% Read"));
    localStorage.setItem("bookreader.book." + cachedbookmeta.bookid + ".curpercent", (book.locations.percentageFromCfi(book.getCurrentLocationCfi()) * 100).toFixed(2));
}

oldcfi = "";
booktimeout = null;
countingtime = true;
booktimeouttime = 5 * 60 * 1000; //5 minutes

addSecs = function (s) {
    if (countingtime) {
        var obs = parseInt(localStorage.getItem("bookreader.book." + cachedbookmeta.bookid + ".secsread") || "0");
        localStorage.setItem("bookreader.book." + cachedbookmeta.bookid + ".secsread", obs + s);

        var ogs = parseInt(localStorage.getItem("bookreader.globalstats.secsread") || "0");
        localStorage.setItem("bookreader.globalstats.secsread", ogs + s);
    }
}

init = function () {
    try {
        bookfilename = atob(getUrlVars()["bookurl"]);
        if (bookfilename.startsWith("file://")) {
            if (!fsExists(bookfilename.replace("file://", ""))) {
                throw {
                    message: "File not found"
                };
            }
        }

        getBookMetadata(bookfilename.replace("file://", ""), true).then(function (meta) {
            cachedbookmeta = meta;
            localStorage.setItem("bookreader.global.lastread.bookid", cachedbookmeta.bookid);
            localStorage.setItem("bookreader.global.lastread.filename", bookfilename);
            localStorage.setItem("bookreader.book." + cachedbookmeta.bookid + ".lastread", new Date().getTime());
            console.log("Updated lastread time to " + localStorage.getItem("bookreader.book." + cachedbookmeta.bookid + ".lastread"));
            document.title = meta.title + " - " + meta.author;
        });

        book = ePub({
            bookPath: null,
            version: 1, // Changing will cause stored Book information to be reloaded
            restore: false, // Skips parsing epub contents, loading from localstorage instead
            storage: false, // true (auto) or false (none) | override: 'ram', 'websqldatabase', 'indexeddb', 'filesystem'
            spreads: true, // Displays two columns
            fixedLayout: false, //-- Will turn off pagination
            styles: {}, // Styles to be applied to epub
            width: false,
            height: false,
        });
        book.open(bookfilename).then(function () {}, function (err) {
            $("#book").empty().html("<div class=\"ui message error\">Error loading book file: " + bookfilename + " . " + err.message + "</div>");
        });

        book.getToc().then(function (toc) {
            var nl = $("#toc");
            nl.empty();
            $.each(toc, function (k, v) {
                console.log((v.cfi || v.href), (v.label || "Chapter " + String(k + 1)));
                nl.append(
                    $("<a class=\"item\" data-url=\"" + (v.cfi || v.href) + "\">" + (v.label || "Chapter " + String(k + 1)) + "</a>")
                    .unbind("click")
                    .click(function () {
                        var u = $(this).attr("data-url");
                        console.log(u);
                        $('#tocbookmarksmodal').modal('hide');
                        // It crashes and uses up all cpu and memory if page changed while modal is open
                        setTimeout(function () {
                            book.goto(u);
                        }, 900);
                    })
                );
            });
            setInterval(function () {
                updateActiveToc();
            }, 1000);
        });

        book.generatePagination(600, 600).then(function () {
            console.log("Pagination generated: " + String(book.pagination.totalPages) + " pages");
            setInterval(function () {
                $(".pagenum").text("Page " + String(book.pagination.pageFromCfi(book.getCurrentLocationCfi()) + " of " + String(book.pagination.totalPages)));
            }, 500);
        });

        setInterval(function () {
            updateBookmarksList();
        }, 1000);

        book.renderTo("book").then(function () {
            setInterval(function () {
                applyBookCSS();
            }, 30000);
            setInterval(function () {
                updateBookmarkIndicator();
            }, 500);
            setInterval(function () {
                updatePercentageIndicator();
            }, 500);
            try {
                applyBookCSS();
            } catch (e) {

            }
            try {
                updateBookmarkIndicator();
            } catch (e) {}

            setTimeout(function () {
                if (localStorage.getItem("bookreader.book." + cachedbookmeta.bookid + ".curposcfi") !== null) {
                    book.goto(localStorage.getItem("bookreader.book." + cachedbookmeta.bookid + ".curposcfi")).then(function () {
                        pageset = true;
                    });
                } else {
                    pageset = true;
                }
                setInterval(function () {
                    if (pageset) {
                        localStorage.setItem("bookreader.book." + cachedbookmeta.bookid + ".curposcfi", book.getCurrentLocationCfi());
                    }
                }, 500);
            }, 500);

            book.locations.generate();
        });
    } catch (err) {
        console.log(err);
        $("#book").empty().html("<div class=\"ui message error\">Error loading book file: " + bookfilename + " . " + err.message + "</div>");
    }

    $("#bookcontainer .prev.nav").click(function () {
        book.prevPage();
    });
    $("#bookcontainer .next.nav").click(function () {
        book.nextPage();
    });


    window.setInterval(function () {
        var newcfi = book.getCurrentLocationCfi();
        if (newcfi !== oldcfi) {
            countingtime = true;
            window.clearTimeout(booktimeout);
            window.setTimeout(function () {
                countingtime = false;
            }, booktimeouttime)
            oldcfi = newcfi;
        }
    }, 1000);

    window.setInterval(function () {
        addSecs(10);
    }, 10000);

    $('.menu .item').tab();
}
init()